package cd.casic.ci.process.api.process.utils;

import cd.casic.ci.process.api.process.constant.ProcessMessageCode;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.regex.Pattern;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.process.api.process.utils
 * @Project：ops-pro
 * @name：DependOnUtils
 * @Date：2025/03/27 14:39
 * @Filename：DependOnUtils
 * @description：Todo
 */
@Slf4j
public class DependOnUtils {

    private static final Pattern REGEX = Pattern.compile("[,;]");

    public static void checkRepeatedJobId(Stage stage) {
        Set<String> jobIdSet = new HashSet<>();
        for (Container c : stage.getContainers()) {
            String jobId = c.getJobId();
            if (jobId == null || jobId.isEmpty()) {
                continue;
            }
            if (jobIdSet.contains(jobId)) {
                String jobName = getContainerName(stage, c, jobId);
                throw new ErrorCodeException(
                        Response.Status.CONFLICT.getStatusCode(),
                        ProcessMessageCode.ERROR_PIPELINE_JOBID_EXIST,
                        new String[]{jobName, jobId},
                        jobName + " 的jobId(" + jobId + ")已存在"
                );
            }
            jobIdSet.add(jobId);
        }
        removeNonexistentJob(stage, jobIdSet);
    }

    private static void removeNonexistentJob(Stage stage, Set<String> jobIdSet) {
        for (Container c : stage.getContainers()) {
            JobControlOption jobControlOption = null;
            if (c instanceof VMBuildContainer) {
                jobControlOption = ((VMBuildContainer) c).getJobControlOption();
            } else if (c instanceof NormalContainer) {
                jobControlOption = ((NormalContainer) c).getJobControlOption();
            }
            if (jobControlOption == null) {
                continue;
            }
            if (jobControlOption.getDependOnType() != DependOnType.ID || jobControlOption.getDependOnId() == null || jobControlOption.getDependOnId().isEmpty()) {
                continue;
            }
            List<String> existJobIds = jobControlOption.getDependOnId().stream()
                    .filter(jobIdSet::contains)
                    .toList();
            jobControlOption.setDependOnId(existJobIds);
        }
    }

    public static void initDependOn(Stage stage, Map<String, String> params) {
        Map<String, Container> allJobId2JobMap = new HashMap<>();
        for (Container c : stage.getContainers()) {
            String jobId = c.getJobId();
            if (jobId == null || jobId.isEmpty()) {
                continue;
            }
            allJobId2JobMap.put(jobId, c);
        }
        if (allJobId2JobMap.isEmpty()) {
            return;
        }

        Map<String, List<String>> cycleCheckJobMap = new HashMap<>();
        for (Container c : stage.getContainers()) {
            JobControlOption jobControlOption = null;
            if (c instanceof VMBuildContainer) {
                jobControlOption = ((VMBuildContainer) c).getJobControlOption();
            } else if (c instanceof NormalContainer) {
                jobControlOption = ((NormalContainer) c).getJobControlOption();
            }
            if (jobControlOption == null) {
                continue;
            }
            List<String> dependOnJobIds = getDependOnJobIds(new DependOnConfig(
                    jobControlOption.getDependOnType(),
                    jobControlOption.getDependOnId(),
                    jobControlOption.getDependOnName()
            ), params);
            if (dependOnJobIds.isEmpty()) {
                continue;
            }
            if (c.getJobId() != null) {
                cycleCheckJobMap.put(c.getJobId(), dependOnJobIds);
            }
            Map<String, String> containerId2JobIds = new HashMap<>();
            for (String dependOnJobId : dependOnJobIds) {
                Container dependOnJob = allJobId2JobMap.get(dependOnJobId);
                if (dependOnJob == null) {
                    continue;
                }
                containerId2JobIds.put(dependOnJob.getId(), dependOnJobId);
            }
            if (!containerId2JobIds.isEmpty()) {
                jobControlOption.setDependOnContainerId2JobIds(containerId2JobIds);
            }
        }

        Map<String, Integer> visited = new HashMap<>();
        for (String jobId : cycleCheckJobMap.keySet()) {
            dsf(jobId, cycleCheckJobMap, visited, stage, allJobId2JobMap);
        }
    }

    public static boolean enableDependOn(Container container) {
        JobControlOption jobControlOption = null;
        if (container instanceof VMBuildContainer) {
            jobControlOption = ((VMBuildContainer) container).getJobControlOption();
        } else if (container instanceof NormalContainer) {
            jobControlOption = ((NormalContainer) container).getJobControlOption();
        }
        if (jobControlOption == null) {
            return false;
        }
        return switch (jobControlOption.getDependOnType()) {
            case ID -> jobControlOption.getDependOnId() != null && !jobControlOption.getDependOnId().isEmpty();
            case NAME -> jobControlOption.getDependOnName() != null && !jobControlOption.getDependOnName().isEmpty();
            default -> false;
        };
    }

    private static List<String> getDependOnJobIds(DependOnConfig dependOnConfig, Map<String, String> params) {
        return switch (dependOnConfig.getDependOnType()) {
            case ID -> dependOnConfig.getDependOnId() != null ? dependOnConfig.getDependOnId() : Collections.emptyList();
            case NAME -> {
                if (dependOnConfig.getDependOnName() == null || dependOnConfig.getDependOnName().isEmpty()) {
                    yield Collections.emptyList();
                } else {
                    String[] dependONames = dependOnConfig.getDependOnName().split(REGEX);
                    List<String> result = new ArrayList<>();
                    for (String name : dependONames) {
                        result.add(EnvUtils.parseEnv(name, params));
                    }
                    yield result;
                }
            }
            default -> Collections.emptyList();
        };
    }

    private static void dsf(String jobId, Map<String, List<String>> dependOnMap, Map<String, Integer> visited, Stage stage, Map<String, Container> allJobId2JobMap) {
        if (visited.getOrDefault(jobId, 0) == 1) {
            String jobName = getContainerName(stage, allJobId2JobMap.get(jobId), jobId);
            String dependJobName = getContainerName(stage, allJobId2JobMap.get(dependOnJobId), dependOnJobId);
            throw new ErrorCodeException(
                    ProcessMessageCode.ERROR_PIPELINE_DEPENDON_CYCLE,
                    new String[]{jobName, dependJobName}
            );
        }
        if (visited.getOrDefault(jobId, 0) == 2) {
            return;
        }

        visited.put(jobId, 1);
        List<String> dependOnJobIds = dependOnMap.getOrDefault(jobId, Collections.emptyList());
        for (String dependOnJobId : dependOnJobIds) {
            dsf(dependOnJobId, dependOnMap, visited, stage, allJobId2JobMap);
        }
        visited.put(jobId, 2);
    }

    private static String getContainerName(Stage stage, Container container, String jobId) {
        if (container == null) {
            return jobId;
        }
        String namePrefix = stage.getName() != null ? stage.getName().replaceFirst("stage-", "") : "";
        return namePrefix + "-" + container.getId();
    }
}
